@@ -1,17 +1,18 @@
 import abc
 import logging
 from random import randrange
+from typing import final, List
 
 if __package__ == "":
-    from tion_btle.tion import tion, TionException
+    from tion_btle.tion import Tion
 else:
-    from .tion import tion, TionException
+    from .tion import Tion
 
 logging.basicConfig(level=logging.DEBUG)
 _LOGGER = logging.getLogger(__name__)
 
 
-class LiteFamily(tion):
+class TionLiteFamily(Tion):
     uuid: str = "98f00001-3788-83ea-453e-f52244709ddb"
     uuid_write: str = "98f00002-3788-83ea-453e-f52244709ddb"
     uuid_notify: str = "98f00003-3788-83ea-453e-f52244709ddb"
@@ -39,24 +40,29 @@ class LiteFamily(tion):
         self._light: bool = False
         self._have_heater: bool = False
 
+    @final
     @property
     def light(self) -> str:
         return self._decode_state(self._light)
 
+    @final
     @light.setter
     def light(self, new_state: str):
         self._light = self._encode_state(new_state)
 
+    @final
     @property
     def random(self) -> bytes:
         # return random hex number.
         return randrange(0xFF)
 
+    @final
     @property
     def random4(self) -> list:
         # return 4 random hex.
         return [self.random, self.random, self.random, self.random]
 
+    @final
     def _collect_message(self, package: bytearray) -> bool:
         self._have_full_package = False
 
@@ -92,35 +98,44 @@ class LiteFamily(tion):
 
         return self._have_full_package
 
-    def _send_request(self, request: bytearray):
+    @final
+    def split_command(self, request: bytearray) -> List[bytearray]:
         def chunks(lst, n):
             """Yield successive n-sized chunks from lst."""
             for j in range(0, len(lst), n):
                 yield lst[j:j + n]
 
+        request.pop()
+
         if len(request) < 20:
-            request[0] = self.SINGLE_PACKET_ID
-            data_for_sent = [request]
-        else:
-            request[0] = self.FIRST_PACKET_ID
-            data_for_sent = list(chunks(request, 20))
+            request.insert(0, self.SINGLE_PACKET_ID)
+            return [request]
 
-            for i in range(1, len(data_for_sent)):
-                if i == len(data_for_sent)-1:
-                    data_for_sent[i].insert(0, self.END_PACKET_ID)
-                else:
-                    data_for_sent[i].insert(0, self.MIDDLE_PACKET_ID)
+        result = list(chunks(request, 19))
 
+        for i in range(0, len(result)):
+            if i == 0:  # First packet
+                result[i].insert(0, self.FIRST_PACKET_ID)
+            elif i == len(result)-1:  # Last packet
+                result[i].insert(0, self.END_PACKET_ID)
+            else:  # Middle packets
+                result[i].insert(0, self.MIDDLE_PACKET_ID)
+
+        return result
+
+    @final
+    async def _send_request(self, request: bytearray):
         self.have_breezer_state = False
 
-        for d in data_for_sent:
+        for d in self.split_command(request):
             _LOGGER.debug("Doing write: request=%s", bytes(d).hex())
-            self._try_write(d)
+            await self._try_write(d)
 
-    def _pair(self):
+    async def _pair(self):
         """Lite family breezers is not require special pairing procedure"""
         return
 
+    @final
     @property
     def CRC(self) -> list:
         return [0xbb, 0xaa]
