@@ -1,17 +1,15 @@
 import logging
 
 if __package__ == "":
-    from tion_btle.tion import tion, TionException
+    from tion_btle.tion import Tion, TionException
 else:
-    from .tion import tion, TionException
-
-from bluepy import btle
+    from .tion import Tion, TionException
 
 logging.basicConfig(level=logging.DEBUG)
 _LOGGER = logging.getLogger(__name__)
 
 
-class S3(tion):
+class TionS3(Tion):
     uuid = "6e400001-b5a3-f393-e0a9-e50e24dcca9e"
     uuid_write = "6e400002-b5a3-f393-e0a9-e50e24dcca9e"
     uuid_notify = "6e400003-b5a3-f393-e0a9-e50e24dcca9e"
@@ -44,9 +42,9 @@ class S3(tion):
     def command_getStatus(self) -> bytearray:
         return self.create_command(self.command_REQUEST_PARAMS)
 
-    def _pair(self):
+    async def _pair(self):
         _LOGGER.debug("Sending pair command")
-        self._send_request(self.pair_command)
+        await self._send_request(self.pair_command)
         _LOGGER.debug("Done!")
 
     def create_command(self, command: int) -> bytearray:
@@ -58,38 +56,6 @@ class S3(tion):
         self._data = package
         return True
 
-    def _get_data_from_breezer(self) -> bytearray:
-        self.have_breezer_state = False
-
-        _LOGGER.debug("Collecting data")
-
-        i = 0
-
-        while i < 10:
-            if self.mac == "dummy":
-                return self._dummy_data
-            else:
-                if self._delegation.haveNewData:
-                    byte_response = self._delegation.data
-                    if self._collect_message(byte_response):
-                        self.have_breezer_state = True
-                        break
-                    i = 0
-                else:
-                    self._btle.waitForNotifications(1.0)
-                i += 1
-        else:
-            _LOGGER.debug("Waiting too long for data")
-            self.notify.read()
-
-        if self.have_breezer_state:
-            result = self._data
-
-        else:
-            raise TionException("_get_data_from_breezer", "Could not get breezer state")
-
-        return result
-
     def _decode_response(self, response: bytearray):
         _LOGGER.debug("Data is %s", bytes(response).hex())
         try:
@@ -130,10 +96,5 @@ class S3(tion):
                     self._encode_status(request["sound"]) << 3)
         return new_settings
 
-    def _send_request(self, request: bytearray):
-        self._try_write(request)
-
-    @property
-    def _dummy_data(self) -> bytearray:
-        return bytearray([0xb3, 0x10, 0x24, 0x14, 0x03, 0x00, 0x15, 0x14, 0x14, 0x8f, 0x00, 0x0c, 0x0a, 0x00, 0x4b,
-                          0x0a, 0x00, 0x33, 0x00, 0x5a])
+    async def _send_request(self, request: bytearray):
+        await self._try_write(request)
