@@ -1,11 +1,13 @@
 import abc
+import asyncio
+import inspect
 import logging
-import time
-from typing import Callable, List
+from asyncio import Semaphore
+from typing import Callable, List, final
 from time import localtime, strftime
 
-from bluepy import btle
-from bluepy.btle import DefaultDelegate
+from bleak import BleakClient
+from bleak import exc
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -16,23 +18,22 @@ class MaxTriesExceededError(Exception):
 
 def retry(retries: int = 2, delay: int = 0):
     def decor(f: Callable):
-        def wrapper(*args, **kwargs):
+        async def wrapper(*args, **kwargs):
             last_info_exception = None
             last_warning_exception = None
             for i in range(retries+1):
                 try:
                     _LOGGER.debug("Trying %d/%d: %s(args=%s,kwargs=%s)", i, retries, f.__name__, args, kwargs)
+                    if inspect.iscoroutinefunction(f):
+                        return await f(*args, **kwargs)
                     return f(*args, **kwargs)
-                except (btle.BTLEDisconnectError, btle.BTLEInternalError) as _e:
-                    _LOGGER.info(f"Got BTLEDisconnectError: {_e}")
-                    last_info_exception = _e
-                except Exception as _e:
+                except (exc.BleakError, exc.BleakDBusError) as _e:
                     next_message = "Will try again" if i < retries else "Will not try again"
                     _LOGGER.warning("Got exception: %s. %s", str(_e), next_message)
                     last_warning_exception = _e
                     pass
                 if delay > 0:
-                    time.sleep(delay)
+                    await asyncio.sleep(delay)
             else:
                 _LOGGER.critical("Retry limit (%d) exceeded for %s(%s, %s)", retries, f.__name__, args, kwargs)
                 if _LOGGER.level > logging.INFO and last_info_exception is not None:
@@ -45,22 +46,14 @@ def retry(retries: int = 2, delay: int = 0):
     return decor
 
 
-class TionDelegation(DefaultDelegate):
+class TionDelegation:
     def __init__(self):
         self._data: List[bytearray] = []
-        self.__topic = None
-        DefaultDelegate.__init__(self)
 
     def handleNotification(self, handle: int, data: bytearray):
         self._data.append(data)
         _LOGGER.debug("Got data in %d response %s", handle, bytes(data).hex())
-        try:
-            self.__topic.read()
-        except btle.BTLEDisconnectError as e:
-            _LOGGER.warning("Got %s while read in handleNotification. May continue working.", str(e))
-
-    def setReadTopic(self, topic):
-        self.__topic = topic
+        _LOGGER.debug(f"{self._data=}")
 
     @property
     def data(self) -> bytearray:
@@ -77,38 +70,7 @@ class TionException(Exception):
         self.message = message
 
 
-class TionDummy:
-    """
-    Class for dummy methods, that should be used for tests
-    """
-
-    _dummy_data: bytearray
-
-    @staticmethod
-    def _connect_dummy(need_notifications: bool = True):
-        """dummy connection"""
-
-        _LOGGER.info("Dummy connect")
-        return
-
-    @staticmethod
-    def _disconnect_dummy():
-        return
-
-    def _get_data_from_breezer_dummy(self) -> bytearray:
-        return self._dummy_data
-
-    @staticmethod
-    def _try_write_dummy(request: bytearray):
-        _LOGGER.debug("Dummy write %s", bytes(request).hex())
-        return
-
-    @staticmethod
-    def _enable_notifications_dummy():
-        return
-
-
-class tion(TionDummy):
+class Tion:
     statuses = ['off', 'on']
     modes = ['recirculation', 'mixed']  # 'recirculation', 'mixed' and 'outside', as Index exception
     uuid_notify: str = ""
@@ -116,7 +78,7 @@ class tion(TionDummy):
 
     def __init__(self, mac: str):
         self._mac = mac
-        self._btle: btle.Peripheral = btle.Peripheral(None)
+        self._btle: BleakClient = BleakClient(self.mac)
         self._delegation = TionDelegation()
         self._fan_speed = 0
         self._model: str = self.__class__.__name__
@@ -131,24 +93,16 @@ class tion(TionDummy):
         self._state: bool = False
         self._heater: bool = False
         self._sound: bool = False
-        self._heating: bool = False
         self._filter_remain: float = 0.0
         self._error_code: int = 0
         self.__failed_connects: int = 0
         self.__connections_count: int = 0
         self.__notifications_enabled: bool = False
         self.have_breezer_state: bool = False
-
-        if self.mac == "dummy":
-            _LOGGER.warning("Dummy mode detected!")
-            self._connect = self._connect_dummy
-            self._disconnect = self._disconnect_dummy
-            self._try_write = self._try_write_dummy
-            self._enable_notifications = self._enable_notifications_dummy
-            self._get_data_from_breezer = self._get_data_from_breezer_dummy
+        self._semaphore = Semaphore(1)
 
     @abc.abstractmethod
-    def _send_request(self, request: bytearray):
+    async def _send_request(self, request: bytearray):
         """ Send request to device
 
         Args:
@@ -209,61 +163,38 @@ class tion(TionDummy):
             "model": self.model,
         }
 
-    def __detect_heating_state(self,
-                               in_temp: int = None,
-                               out_temp: int = None,
-                               heater_temp: int = None,
-                               heater: str = None) -> None:
-        """
-        Tries to guess is heater working right now
-        :param in_temp: air intake temperature
-        :param out_temp: ait outtake temperature
-        :param heater_temp: target temperature for heater
-        :param heater: heater state
-        :return: None
-        """
-        if in_temp is None:
-            in_temp = self.in_temp
-        if out_temp is None:
-            out_temp = self.out_temp
-        if heater_temp is None:
-            heater_temp = self.heater_temp
-        if heater is None:
-            heater = self.heater
+    @final
+    @property
+    def heating(self) -> str:
+        """Tries to guess is heater working right now."""
+        if self.heater == "off":
+            return "off"
 
-        if heater == "off":
-            self.heating = "off"
-        else:
-            if heater_temp - in_temp > 3 and out_temp > in_temp:
-                self.heating = "on"
-            else:
-                self.heating = "off"
+        if self.heater_temp - self.in_temp > 3 and self.out_temp > self.in_temp:
+            return "on"
 
-    def get_state_from_breezer(self, keep_connection: bool = False) -> None:
+        return "off"
+
+    @final
+    async def get_state_from_breezer(self) -> None:
         """
         Get current state from breezer
-        :param keep_connection: should we keep connection to device or disconnect after getting data
         :return: None
         """
         try:
-            self.connect()
-            self._try_write(request=self.command_getStatus)
-            response = self._get_data_from_breezer()
+            await self.connect()
+            await self._try_write(request=self.command_getStatus)
+            response = await self._get_data_from_breezer()
         finally:
-            if not keep_connection:
-                self.disconnect()
-            else:
-                _LOGGER.warning("You are using keep_connection parameter of get method. It will be removed in v2.0.0")
-                self.__connections_count -= 1
+            await self.disconnect()
 
         self._decode_response(response)
-        self.__detect_heating_state()
 
-    def get(self, keep_connection: bool = False, skip_update: bool = False) -> dict:
+    @final
+    async def get(self, skip_update: bool = False) -> dict:
         """
         Report current breezer state
         :param skip_update: may we skip requesting data from breezer or not
-        :param keep_connection: should we keep connection to device or disconnect after getting data
         :return:
           dictionary with device state
         """
@@ -271,12 +202,13 @@ class tion(TionDummy):
             _LOGGER.debug(f"Skipping getting state from breezer because skip_update={skip_update} and "
                           f"have_breezer_state={self.have_breezer_state}")
         else:
-            self.get_state_from_breezer(keep_connection)
+            await self.get_state_from_breezer()
         common = self.__generate_common_json()
         model_specific_data = self._generate_model_specific_json()
 
         return {**common, **model_specific_data}
 
+    @final
     def _set_internal_state_from_request(self, request: dict) -> None:
         """
         Set internal parameters based on user request
@@ -291,7 +223,8 @@ class tion(TionDummy):
             except KeyError:
                 pass
 
-    def set(self, new_settings=None) -> None:
+    @final
+    async def set(self, new_settings=None) -> None:
         """
         Set new breezer state
         :param new_settings: json with new state
@@ -308,18 +241,20 @@ class tion(TionDummy):
             pass
 
         try:
-            self.connect()
-            current_settings = self.get(skip_update=True)
+            await self.connect()
+            current_settings = await self.get(skip_update=True)
 
             merged_settings = {**current_settings, **new_settings}
 
             encoded_request = self._encode_request(merged_settings)
             _LOGGER.debug("Will write %s", encoded_request)
-            self._send_request(encoded_request)
+            await self._send_request(encoded_request)
             self._set_internal_state_from_request(new_settings)
+            await self._get_data_from_breezer()
         finally:
-            self.disconnect()
+            await self.disconnect()
 
+    @final
     @property
     def mac(self):
         return self._mac
@@ -335,6 +270,7 @@ class tion(TionDummy):
         barrier = 0b10000000
         return raw if raw < barrier else -(~(raw - barrier) + barrier + 1)
 
+    @final
     def _process_status(self, code: int) -> str:
         try:
             status = self.statuses[code]
@@ -342,89 +278,66 @@ class tion(TionDummy):
             status = 'unknown'
         return status
 
+    @final
     @property
     def connection_status(self):
-        connection_status = "disc"
-        try:
-            connection_status = self._btle.getState()
-        except btle.BTLEInternalError as e:
-            if str(e) == "Helper not started (did you call connect()?)":
-                pass
-            else:
-                raise e
-        except btle.BTLEDisconnectError as e:
-            pass
-        except BrokenPipeError as e:
-            self._btle = btle.Peripheral(None)
-
-        return connection_status
+        status = "connected" if self._btle.is_connected else "disc"
+        return status
 
+    @final
     @retry(retries=1, delay=2)
-    def _try_connect(self) -> None:
+    async def _try_connect(self) -> bool:
         """Tries to connect with retries"""
 
-        self._btle.connect(self.mac, btle.ADDR_TYPE_RANDOM)
+        return await self._btle.connect()
 
-    def _connect(self, need_notifications: bool = True):
-        _LOGGER.debug("Connecting")
+    @final
+    async def _connect(self, need_notifications: bool = True):
+        _LOGGER.debug(f"Connecting. {self.connection_status=}.")
         if self.connection_status == "disc":
-            self._try_connect()
-            for tc in self._btle.getCharacteristics():
-                if tc.uuid == self.uuid_notify:
-                    self.notify = tc
-                if tc.uuid == self.uuid_write:
-                    self.write = tc
+            try:
+                await self._try_connect()
+            except exc.BleakError as e:
+                _LOGGER.warning(f"Got {str(e)=} exception in _connect")
+                raise e
+
             if need_notifications:
-                self._enable_notifications()
+                await self._enable_notifications()
             else:
                 _LOGGER.debug("Notifications was not requested")
+        _LOGGER.debug(f"_connect done. {self.connection_status=}.")
 
-    def _disconnect(self):
+    @final
+    async def _disconnect(self):
+        _LOGGER.debug(f"Disconnecting. {self.connection_status=}.")
         if self.connection_status != "disc":
-            self._btle.disconnect()
+            await self._btle.disconnect()
+        _LOGGER.debug(f"_disconnect done. {self.connection_status=}")
 
+    @final
     @retry(retries=3)
-    def _try_write(self, request: bytearray):
-        _LOGGER.debug("Writing %s to %s", bytes(request).hex(), self.write.uuid)
-        return self.write.write(request)
+    async def _try_write(self, request: bytearray):
+        _LOGGER.debug(f"Writing {bytes(request).hex()} to {self.uuid_write}, {self.connection_status=}")
+        return await self._btle.write_gatt_char(
+            self.uuid_write,
+            request,
+            False
+        )
 
-    def __write_to_notify_handle(self, data):
-        need_response: bool = True if self.model == "Lite" else False
-        _LOGGER.debug("Notify handler is %s", self.notify.getHandle())
-        notify_handle = self.notify.getHandle() + 1
-
-        _LOGGER.debug("Will write %s to %s handle with withResponse=%s", data, notify_handle, need_response)
-        result = self._btle.writeCharacteristic(notify_handle, data, withResponse=need_response)
-        _LOGGER.debug("Result is %s", result)
-
-    def _enable_notifications(self):
-        _LOGGER.debug("Enabling notification")
-        setup_data = b"\x01\x00"
-
-        self.__write_to_notify_handle(setup_data)
-
-        self._btle.withDelegate(self._delegation)
-        _LOGGER.debug("Delegation enabled")
+    @final
+    async def _enable_notifications(self):
+        _LOGGER.debug(f"Enabling notification. {self.connection_status=}")
         try:
-            data = self.notify.read()
-            _LOGGER.debug("First read done. Data is %s", bytes(data).hex())
-        except btle.BTLEDisconnectError as e:
-            _LOGGER.critical("_enable_notifications: got '%s' while first read! Could not continue!", str(e))
+            await self._btle.start_notify(self.uuid_notify, self._delegation.handleNotification)
+        except exc.BleakError as e:
+            _LOGGER.warning("Got exception %s while enabling notifications!" % str(e))
             raise e
 
-        self._delegation.setReadTopic(self.notify)
-        _LOGGER.debug("enable_notification is done")
         self.__notifications_enabled = True
+        _LOGGER.debug(f"_enable_notifications done")
+        return
 
-    def _disable_notifications(self):
-        _LOGGER.debug("Disabling notifications")
-        setup_data = b"\x00\x00"
-
-        self.__write_to_notify_handle(setup_data)
-
-        _LOGGER.debug("disable_notification is done")
-        self.__notifications_enabled = False
-
+    @final
     @property
     def fan_speed(self):
         return self._fan_speed
@@ -440,6 +353,7 @@ class tion(TionDummy):
 
         # self.set({"fan_speed": new_speed})
 
+    @final
     def _process_mode(self, mode_code: int) -> str:
         try:
             mode = self.modes[mode_code]
@@ -455,80 +369,89 @@ class tion(TionDummy):
     def _encode_state(state: str) -> bool:
         return state == "on"
 
+    @final
     @property
     def state(self) -> str:
         return self._decode_state(self._state)
 
+    @final
     @state.setter
     def state(self, new_state: str):
         self._state = self._encode_state(new_state)
 
+    @final
     @property
     def heater(self) -> str:
         return self._decode_state(self._heater)
 
+    @final
     @heater.setter
     def heater(self, new_state: str):
         self._heater = self._encode_state(new_state)
 
+    @final
     @property
     def heater_temp(self) -> int:
         return self._heater_temp
 
+    @final
     @heater_temp.setter
     def heater_temp(self, new_temp: int):
         self._heater_temp = new_temp
 
+    @final
     @property
     def target_temp(self) -> int:
         return self.heater_temp
 
+    @final
     @target_temp.setter
     def target_temp(self, new_temp: int):
         self.heater_temp = new_temp
 
+    @final
     @property
     def in_temp(self):
         """Income air temperature"""
         return self._in_temp
 
+    @final
     @property
     def out_temp(self):
         """Outcome air temperature"""
         return self._out_temp
 
+    @final
     @property
     def sound(self) -> str:
         return self._decode_state(self._sound)
 
+    @final
     @sound.setter
     def sound(self, new_state: str):
         self._sound = self._encode_state(new_state)
 
+    @final
     @property
     def filter_remain(self) -> float:
         return self._filter_remain
 
-    @property
-    def heating(self) -> str:
-        return self._decode_state(self._heating)
-
-    @heating.setter
-    def heating(self, new_state: str):
-        self._heating = self._encode_state(new_state)
-
+    @final
     @property
     def mode(self):
         return self._process_mode(self._mode)
 
+    @final
     @mode.setter
     def mode(self, new_state: str):
         self._mode = self._encode_mode(new_state)
 
+    @final
     @property
     def model(self) -> str:
-        return self._model
+        return self._model.removeprefix("Tion")
 
+    @final
     def _encode_status(self, status: str) -> int:
         """
         Encode string status () to int
@@ -537,6 +460,7 @@ class tion(TionDummy):
         """
         return self.statuses.index(status) if status in self.statuses else 0
 
+    @final
     def _encode_mode(self, mode: str) -> int:
         """
         Encode string mode to integer
@@ -545,65 +469,48 @@ class tion(TionDummy):
         """
         return self.modes.index(mode) if mode in self.modes else 2
 
-    def pair(self):
+    @final
+    async def pair(self):
         _LOGGER.debug("Pairing")
-        self._connect(need_notifications=False)
+        await self._connect(need_notifications=False)
         _LOGGER.debug("Connected. BT pairing ...")
         try:
-            # use private methods to avoid disconnect if already paired
-            self._btle._writeCmd('pair' + '\n')
-            rsp = self._btle._waitResp('mgmt')
-            _LOGGER.debug("Got response while sending pair command: %s", rsp)
-            try:
-                estat = rsp['estat'][0]
-            except KeyError:
-                # we may have no estat in response. It is OK.
-                estat = 0
-
-            if estat == 20:
-                # it seem in https://github.com/TionAPI/tion_python/issues/17 that we may ignore it.
-                _LOGGER.warning("bt pairing: could not pair! Permission denied. Check permissions at host. "
-                                "Ignore it if all goes well")
-            elif estat == 0 or estat == 19 or rsp['code'][0] == 'success':
-                # 0 -- fine, no errors; 19 -- paired.
-                try:
-                    msg = rsp['emsg'][0]
-                except KeyError:
-                    msg = rsp['code'][0]
-                _LOGGER.debug(msg)
-            else:
-                _LOGGER.critical("Unexpected response: %s", rsp)
-                raise TionException('pair', rsp)
+            await self._btle.pair()
             # device-specific pairing
             _LOGGER.debug("Device-specific pairing ...")
-            self._pair()
+            await self._pair()
             _LOGGER.debug("Device pair is done")
         except Exception as e:
             _LOGGER.critical(f"Got exception while pair {type(e).__name__}: {str(e)}")
             raise TionException('pair', f"{type(e).__name__}: {str(e)}")
         finally:
             _LOGGER.debug("disconnected")
-            self._disconnect()
+            await self._disconnect()
 
     @abc.abstractmethod
-    def _pair(self):
+    async def _pair(self):
         """Perform model-specific pair steps"""
 
-    def connect(self):
+    @final
+    async def connect(self):
         if self.__connections_count < 0:
             self.__connections_count = 0
 
         if self.__connections_count == 0:
             self.have_breezer_state = False
-            self._connect()
+            async with self._semaphore:
+                await self._connect()
 
         self.__connections_count += 1
 
-    def disconnect(self):
+    @final
+    async def disconnect(self):
         self.__connections_count -= 1
         if self.__connections_count <= 0:
-            self._disconnect()
+            await self._disconnect()
             self.have_breezer_state = False
+            while self._delegation.haveNewData:
+                _LOGGER.debug(f"Cleaning data in disconnect: {self._delegation.data=}")
 
     @property
     @abc.abstractmethod
@@ -621,15 +528,8 @@ class tion(TionDummy):
         """
         raise NotImplementedError()
 
-    @property
-    @abc.abstractmethod
-    def _dummy_data(self) -> bytearray:
-        """
-        full response from breezer for test, as it was collected from packages: without deader and CRC
-        """
-        raise NotImplementedError()
-
-    def _get_data_from_breezer(self) -> bytearray:
+    @final
+    async def _get_data_from_breezer(self) -> bytearray:
         """ Get byte array with breezer response on state request
 
         :returns:
@@ -642,21 +542,17 @@ class tion(TionDummy):
         i = 0
 
         while i < 10:
-            if self.mac == "dummy":
-                return self._dummy_data
+            if self._delegation.haveNewData:
+                byte_response = self._delegation.data
+                if self._collect_message(byte_response):
+                    self.have_breezer_state = True
+                    break
+                i = 0
             else:
-                if self._delegation.haveNewData:
-                    byte_response = self._delegation.data
-                    if self._collect_message(byte_response):
-                        self.have_breezer_state = True
-                        break
-                    i = 0
-                else:
-                    self._btle.waitForNotifications(1.0)
-                i += 1
+                await asyncio.sleep(1)
+            i += 1
         else:
             _LOGGER.debug("Waiting too long for data")
-            self.notify.read()
 
         if self.have_breezer_state:
             result = self._data
@@ -664,4 +560,4 @@ class tion(TionDummy):
         else:
             raise TionException("_get_data_from_breezer", "Could not get breezer state")
 
-        return result
\ No newline at end of file
+        return result
